/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * The origin of this IDL file is
 * http://dvcs.w3.org/hg/speech-api/raw-file/tip/speechapi.html
 *
 * Copyright © 2012 W3C® (MIT, ERCIM, Keio), All Rights Reserved. W3C
 * liability, trademark and document use rules apply.
 */

#include "nsIDOMEvent.idl"

interface SpeechRecognitionResultList;

[scriptable, builtinclass, uuid(98dded70-33af-42d5-819d-e15b6f4a3aba)]
interface nsIDOMSpeechRecognitionEvent : nsIDOMEvent {
    [noscript] void initSpeechRecognitionEvent(in DOMString eventTypeArg,
                                               in boolean canBubbleArg,
                                               in boolean cancelableArg,
                                               in unsigned long resultIndex,
                                               in nsISupports results,
                                               in DOMString interpretation,
                                               in nsIDOMDocument emma);

    readonly attribute unsigned long resultIndex;
    readonly attribute nsISupports results;
    readonly attribute DOMString interpretation;
    readonly attribute nsIDOMDocument emma;
};
