/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsISimpleEnumerator;
interface nsIURI;

/**
 * The chrome registry implements this interface to give overlays
 * to the gecko XUL engine.
 */

[scriptable, uuid(1d5b5b94-dc47-4050-93b7-ac092e383cad)]
interface nsIXULOverlayProvider : nsISupports
{
    /**
     * Get the XUL overlays for a particular chrome URI.
     *
     * @param aURI  The URI being loaded
     * @return      An enumerator of nsIURI for the overlays of this URI 
     */
    nsISimpleEnumerator /*nsIURI*/ getXULOverlays(in nsIURI aURI);

    /**
     * Get the style overlays for a particular chrome URI.
     *
     * @param aURI  The URI being loaded
     * @return      An enumerator of nsIURI for the overlays of this URI 
     */
    nsISimpleEnumerator /*nsIURI*/ getStyleOverlays(in nsIURI aURI);
};
